/*
 * arch/arm/mach-ns9xxx/board-a9m9750dev.c
 *
 * Copyright (C) 2006 by Digi International Inc.
 * All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License version 2 as published by
 * the Free Software Foundation.
 */
#include <linux/platform_device.h>
#include <linux/interrupt.h>
#include <linux/spi/spi.h>
#include <linux/spi/ads7846.h>

#include <asm/arch/board.h>
#include <asm/arch/hardware.h>
#include <asm/arch/regs-bbus.h>
#include <asm/arch/spi.h>

#include "processor-ns9360.h"

#if defined CONFIG_MACH_CC9P9360JS
#define	JSDEV_GPIO_TS_PENIRQ	(69)
#define	JSDEV_GPIO_TS_IRQ	IRQ_EXT1
#else
#define	JSDEV_GPIO_TS_PENIRQ	(18)
#define	JSDEV_GPIO_TS_IRQ	IRQ_EXT3
#endif

#if defined(CONFIG_TOUCHSCREEN_ADS7846) || defined(CONFIG_TOUCHSCREEN_ADS7846_MODULE)
static int jumpstart_get_pendown_state(void)
{
	return ns9xxx_gpio_getpin(JSDEV_GPIO_TS_PENIRQ) ? 0 : 1;
}

static struct ads7846_platform_data cc9c_ads7843_platform_data = {
	.x_min			= CONFIG_TS_X_MIN,
	.y_min			= CONFIG_TS_Y_MIN,
	.x_max			= CONFIG_TS_X_MAX,
	.y_max			= CONFIG_TS_Y_MAX,
	.rotate			= 180,
	.buflen			= 20,
	.skip_samples		= 0,
	.model			= 7843,
	.get_pendown_state	= jumpstart_get_pendown_state,

};

static struct spi_board_info cc9c_spi_board_info[] __initdata = {
	[0] = {
		.modalias       = "ads7846",
		.bus_num        = 0,
		.chip_select    = 0,
		.max_speed_hz   = 200000,
		.irq		= JSDEV_GPIO_TS_IRQ,
		.platform_data	= &cc9c_ads7843_platform_data,
	},
};
#else
static struct spi_board_info cc9c_spi_board_info[] __initdata = {};
#endif



void __init board_jumpstart_setup_touch_if(void)
{
	int ret;

	ret = ns9xxx_request_gpio(JSDEV_GPIO_TS_PENIRQ, "ads7846");
	if (ret != 0) {
		printk(KERN_WARNING "Unable to request GPIO %d for ts PENIRQ\n",
			JSDEV_GPIO_TS_PENIRQ);
		/* Avoid ads7846 driver load */
		cc9c_spi_board_info[0].irq = NO_IRQ;
	} else {
		(void)ns9xxx_gpio_cfgpin(JSDEV_GPIO_TS_PENIRQ, GPIO_CFG_FUNC_2);
		set_irq_type(JSDEV_GPIO_TS_IRQ, IRQT_FALLING);
	}
}


void __init board_jumpstart_init_machine(void)
{
	board_jumpstart_setup_touch_if();
	spi_register_board_info(cc9c_spi_board_info, ARRAY_SIZE(cc9c_spi_board_info));
	ns9xxx_register_spi_devices();
}


void __init board_jumpstart_fixup(void)
{
	__board_type = BOARD_TYPE_JUMPSTART;

	board_init_machine = board_jumpstart_init_machine;
}
