/*
 * arch/arm/mach-ns9xxx/generic.c
 *
 * Copyright (C) 2006 by Digi International Inc.
 * All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License version 2 as published by
 * the Free Software Foundation.
 */
#include <linux/kernel.h>
#include <linux/init.h>
#include <linux/platform_device.h>
#include <asm/memory.h>
#include <asm/page.h>
#include <asm/mach-types.h>
#include <asm/mach/map.h>
#include <asm/arch/regs-sys.h>
#include <asm/arch/regs-mem.h>
#include <asm/arch/board.h>
#include <asm/arch/regs-usb.h>
#include <asm/arch/irqs.h>

#include "processor-ns9360.h"
#include <asm/arch/ns9xxx_udc.h>
#include <asm/arch/regs-bbus.h>

spinlock_t ns_gpio_lock;	/* use it to lock all GPIO accesses */
EXPORT_SYMBOL( ns_gpio_lock );

unsigned int __board_type;
unsigned int __processor_type;

void (*module_map_io)(void) __initdata;
void (*board_map_io)(void) __initdata;
void (*board_init_machine)(void) __initdata;

static struct map_desc standard_io_desc[] __initdata = {
	{ /* BBus */
		.virtual = io_p2v(0x90000000),
		.pfn = __phys_to_pfn(0x90000000),
		.length = 0x00900000,
		.type = MT_DEVICE,
	}, { /* AHB */
		.virtual = io_p2v(0xa0100000),
		.pfn = __phys_to_pfn(0xa0100000),
		.length = 0x00900000,
		.type = MT_DEVICE,
	},
};

static struct resource ns9xxx_usbhost_resources [] = {
	[0] = {
		.start  = USB_MODULE_BASE_PA,
		.end    = USB_MODULE_BASE_PA + 0x1fff,
		.flags  = IORESOURCE_MEM,
	},
	[1] = {
		.start = (IRQ_USBHOST),
		.end = (IRQ_USBHOST),
		.flags = IORESOURCE_IRQ,
	},
};

static u64 ns9xxx_usbhost_dma_mask = 0xffffffffUL;

static struct resource ns9xxx_usbdevice_resources [] = {
	[0] = {
		.start  = USB_DEVICE_PA,
		.end    = USB_DEVICE_PA_END,
		.flags  = IORESOURCE_MEM,
	},
	[1] = {
		.start = USB_DEVICE_DMA_PA,
		.end   = USB_DEVICE_DMA_PA_END,
		.flags = IORESOURCE_MEM,
	},
	[2] = {
		.start = IRQ_USBDEVICE,
		.end   = IRQ_USBDEVICE,
		.flags = IORESOURCE_IRQ,
	},
};

int udc_is_connected(void)
{
	/* We have no gpio to detect connection state on the dev board */
	return 1;
}

void udc_command(int command)
{
	if(command == NS9XXX_UDC_CMD_CONNECT) {
		ns9xxx_gpio_setpin(GPIO_UDC_ENUM, 1);
	} else {
		ns9xxx_gpio_setpin(GPIO_UDC_ENUM, 0);
	}
}

static struct ns9xxx_udc_mach_info ns9xxx_udc_platform_data = {
	.udc_is_connected = udc_is_connected,
	.udc_command = udc_command,
};

static u64 ns9xxx_usbdevice_dma_mask = 0xffffffffUL;

static struct platform_device ns9xxx_udc = {
	.name		= "ns9xxx-udc",
	.id		= 0,
	.dev		= {
		.dma_mask = &ns9xxx_usbdevice_dma_mask,
		.coherent_dma_mask = 0xffffffffUL,
		.platform_data = &ns9xxx_udc_platform_data,
	},
	.num_resources	= ARRAY_SIZE(ns9xxx_usbdevice_resources),
	.resource	= ns9xxx_usbdevice_resources,
};

/* Watchdog */
static struct resource ns9xxx_wdt_resources[] = {
	[0] = {
		.start = SYS_SWC_BASE_PA,
		.end   = SYS_SWC_BASE_PA + SYS_SWC_REG_LEN - 1,
		.flags = IORESOURCE_MEM,
	},
	[1] = {
		.start = IRQ_WATCHDOG,
		.end   = IRQ_WATCHDOG,
		.flags = IORESOURCE_IRQ,
	}

};

struct platform_device ns9xxx_device_wdt = {
	.name		  = "ns9xxx_wdt",
	.id		  = -1,
	.num_resources	  = ARRAY_SIZE(ns9xxx_wdt_resources),
	.resource	  = ns9xxx_wdt_resources,
};

EXPORT_SYMBOL(ns9xxx_device_wdt);


static struct platform_device ns9xxx_usbhost = {
	.name           = "ns9xxx-ohci",
	.id             = -1,
	.dev            = {
		.dma_mask = &ns9xxx_usbhost_dma_mask,
		.coherent_dma_mask = 0xffffffffUL,
	},
	.num_resources  = ARRAY_SIZE(ns9xxx_usbhost_resources),
	.resource       = ns9xxx_usbhost_resources,
};

void __init ns9xxx_map_io(void)
{
	iotable_init(standard_io_desc, ARRAY_SIZE(standard_io_desc));

	if (board_map_io)
		board_map_io();

	if (module_map_io)
		module_map_io();
}

void __init ns9xxx_init_machine(void)
{
	MEM_SMEW = 0x10;

	spin_lock_init( &ns_gpio_lock );

	if (board_init_machine)
		board_init_machine();

	platform_device_register(&ns9xxx_device_wdt);
	platform_device_register(&ns9xxx_usbhost);
	platform_device_register(&ns9xxx_udc);

#ifdef CONFIG_USB_GADGET_NS9360_EXT_MAX3454E
	if (ns9xxx_request_gpio( GPIO_UDC_ENUM, "usb-device-ext-ENUM") == 0) {
		ns9xxx_gpio_cfgpin( GPIO_UDC_ENUM, GPIO_CFG_OUTPUT );
		ns9xxx_gpio_setpin( GPIO_UDC_ENUM, 0 );
		ns9xxx_release_gpio( GPIO_UDC_ENUM );
	}
#endif
}
