/*
 * arch/arm/mach-ns9xxx/irq.c
 *
 * Copyright (C) 2006 by Digi International Inc.
 * All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License version 2 as published by
 * the Free Software Foundation.
 */

#include <linux/interrupt.h>
#include <asm/mach/irq.h>
#include <asm/mach-types.h>
#include <asm/arch-ns9xxx/regs-sys.h>
#include <asm/arch-ns9xxx/regs-bbus.h>
#include <asm/arch-ns9xxx/regs-bbus_bridge.h>
#include <asm/arch-ns9xxx/irqs.h>
#include <asm/arch-ns9xxx/board.h>
#include <asm/arch-ns9xxx/regs-usb.h>
#include "generic.h"

#include <linux/kallsyms.h>

static inline void ns9xxx_ack_ahb_irq(unsigned int irq);
static inline void ns9xxx_mask_ahb_irq(unsigned int irq);
static inline void ns9xxx_unmask_ahb_irq(unsigned int irq);
static int nx9xxx_irq_type(unsigned int irq, unsigned int type);
static inline void ns9xxx_ack_bbus_irq(unsigned int irq);
static inline void ns9xxx_mask_bbus_irq(unsigned int irq);
static inline void ns9xxx_unmask_bbus_irq(unsigned int irq);
static inline void ns9xxx_mask_bbus_dma_irq(unsigned int irq);
static inline void ns9xxx_unmask_bbus_dma_irq( unsigned int irq );
static inline void ns9xxx_ack_bbus_dma_irq(unsigned int irq);
static inline void ns9xxx_mask_usb_irq(unsigned int irq);
static inline void ns9xxx_unmask_usb_irq(unsigned int irq);
static inline void ns9xxx_ack_usb_irq(unsigned int irq);






void (*board_init_irq)(void) __initdata;

static struct irqchip ns9xxx_ahb_chip = {
	.ack	  = ns9xxx_ack_ahb_irq,
	.mask	  = ns9xxx_mask_ahb_irq,
	.unmask   = ns9xxx_unmask_ahb_irq,
	.set_type = nx9xxx_irq_type,

};

static inline void ns9xxx_ack_ahb_irq(unsigned int irq)
{
	BUG_ON(irq > MAX_AHB_IRQ);

	ns9xxx_mask_ahb_irq(irq);
	SYS_ISRADDR = 0;
}

static inline void ns9xxx_mask_ahb_irq(unsigned int irq)
{
	BUG_ON(irq > MAX_AHB_IRQ);

	/* XXX: better use cpp symbols */
	SYS_IC(irq / 4) &= ~(1 << (7 + 8 * (3 - (irq & 3))));
}

static inline void ns9xxx_unmask_ahb_irq(unsigned int irq)
{
	BUG_ON(irq > MAX_AHB_IRQ);

	/* XXX: better use cpp symbols */
	SYS_IC(irq / 4) |= 1 << (7 + 8 * (3 - (irq & 3)));
}

static int nx9xxx_irq_type(unsigned int irq, unsigned int type)
{
	u32 eicval;

	if ((irq < IRQ_EXT0) || (irq > IRQ_EXT3))
		return -EINVAL;

	switch (type) {
		case IRQT_HIGH:
			eicval = SYS_EIC_LVEDG_LEVEL | SYS_EIC_PLTY_AH;
			break;

		case IRQT_LOW:
			eicval = SYS_EIC_LVEDG_LEVEL | SYS_EIC_PLTY_AL;
			break;

		case IRQT_RISING:
			eicval = SYS_EIC_LVEDG_EDGE | SYS_EIC_PLTY_AH;
			break;

		case IRQT_FALLING:
			eicval = SYS_EIC_LVEDG_EDGE | SYS_EIC_PLTY_AL;
			break;

		default:
			return -EINVAL;
	}

	SYS_EIC(irq - IRQ_EXT0) = eicval;	
	return 0;
}


static struct irqchip ns9xxx_bbus_chip = {
	.ack	= ns9xxx_ack_bbus_irq,
	.mask	= ns9xxx_mask_bbus_irq,
	.unmask = ns9xxx_unmask_bbus_irq
};

static inline void ns9xxx_ack_bbus_irq(unsigned int irq)
{
	ns9xxx_mask_bbus_irq( irq );
	ns9xxx_mask_ahb_irq( IRQ_BBUSAGG );
	SYS_ISRADDR = 0; /* [1] p. 125 */
}

static inline void ns9xxx_mask_bbus_irq(unsigned int irq)
{
	BUG_ON((irq < BBUS_IRQ(0)) || (irq > MAX_BBUS_IRQ));

	BBUS_IER &= ~(1 << (irq - BBUS_IRQ(0)));
}

static inline void ns9xxx_unmask_bbus_irq(unsigned int irq)
{
	BUG_ON((irq < BBUS_IRQ(0)) || (irq > MAX_BBUS_IRQ));

	BBUS_IER |= 1 << (irq - BBUS_IRQ(0));
	ns9xxx_unmask_ahb_irq(IRQ_BBUSAGG);
}


static struct irqchip ns9xxx_bbus_dma_chip = {
	.ack	= ns9xxx_ack_bbus_dma_irq,
	.mask	= ns9xxx_mask_bbus_dma_irq,
	.unmask = ns9xxx_unmask_bbus_dma_irq
};

static inline void ns9xxx_ack_bbus_dma_irq(unsigned int irq)
{
	ns9xxx_mask_bbus_dma_irq(irq);
	SYS_ISRADDR = 0; /* [1] p. 125 */
}

static inline void ns9xxx_mask_bbus_dma_irq(unsigned int irq)
{
	BUG_ON((irq < BBUS_DMA_IRQ(0)) || (irq > MAX_BBUS_DMA_IRQ));

	BBUS_DMA_IEN &= ~(1 << (irq - BBUS_DMA_IRQ(0)));
}

static inline void ns9xxx_unmask_bbus_dma_irq( unsigned int irq )
{
	BUG_ON((irq < BBUS_DMA_IRQ(0)) || (irq > MAX_BBUS_DMA_IRQ));

	BBUS_DMA_IEN |= 1 << (irq - BBUS_DMA_IRQ(0));
}

static struct irqchip ns9xxx_usb_chip = {
	.ack	= ns9xxx_ack_usb_irq,
	.mask	= ns9xxx_mask_usb_irq,
	.unmask = ns9xxx_unmask_usb_irq
};

static inline void ns9xxx_mask_usb_irq(unsigned int irq)
{
	unsigned int mask = 0;

	BUG_ON((irq < USB_IRQ(0)) || (irq > MAX_USB_IRQ));

	/* don't allow selection of individual DMA interrupts. */
	if (irq == IRQ_USB_DMA)
		mask = NS_USB_INT_DMA_ALL;
	else if (irq == IRQ_USB_FIFO)
		mask = NS_USB_INT_FIFO_ALL;
	else if (irq == IRQ_USB_HOST)
		mask = NS_USB_INT_HOST;

	USB_IEN = (USB_IEN & ~mask) | NS_USB_INT_GBL_EN;
}

static inline void ns9xxx_unmask_usb_irq(unsigned int irq)
{
	unsigned int mask = 0;

	BUG_ON((irq < USB_IRQ(0)) || (irq > MAX_USB_IRQ));

	/* don't allow selection of individual DMA interrupts. */
	if (irq == IRQ_USB_DMA)
		mask = NS_USB_INT_DMA_ALL;
	else if (irq == IRQ_USB_FIFO)
		mask = NS_USB_INT_FIFO_ALL;
	else if (irq == IRQ_USB_HOST)
		mask = NS_USB_INT_HOST;

	USB_IEN |= (mask | NS_USB_INT_GBL_EN);
}

static inline void ns9xxx_ack_usb_irq(unsigned int irq)
{
	ns9xxx_mask_usb_irq(irq);
	SYS_ISRADDR = 0; /* [1] p. 125 */
}





void __init ns9xxx_init_irq(void)
{
	int i;

	/* disable all IRQs */
	for (i = 0; i < 8; ++i)
		SYS_IC(i) = (4 * i) << 24 | (4 * i + 1) << 16 |
			(4 * i + 2) << 8 | (4 * i + 3);

	/* simple interrupt prio table: prio(x) < prio(y) <=> x < y */
	for (i = 0; i < 32; ++i)
		SYS_IVA(i) = i;

	/* set up the AHB interrupt handlers */
	for (i = AHB_IRQ(0); i < BBUS_IRQ(0); i++) {
		set_irq_chip(i, &ns9xxx_ahb_chip);
		set_irq_handler(i, do_level_IRQ);
		set_irq_flags(i, IRQF_VALID);
	}

	/* set up the BBUS interrupt handlers */
	for (i = BBUS_IRQ(0); i < BBUS_DMA_IRQ(0); i++) {
		set_irq_chip(i, &ns9xxx_bbus_chip);
		set_irq_handler(i, do_level_IRQ);
		set_irq_flags(i, IRQF_VALID);
	}

	/* set up the BBUS DMA interrupt handlers */
	for (i = BBUS_DMA_IRQ(0); i < USB_IRQ(0); i++) {
		set_irq_chip(i, &ns9xxx_bbus_dma_chip);
		set_irq_handler(i, do_level_IRQ);
		set_irq_flags(i, IRQF_VALID);
	}

	/* set up the USB interrupt handlers */
	for (i = USB_IRQ(0); i <= MAX_USB_IRQ; i++) {
		set_irq_chip(i, &ns9xxx_usb_chip);
		set_irq_handler(i, do_level_IRQ);
		set_irq_flags(i, IRQF_VALID | IRQF_PROBE);
	}

	/* 
	 * XXX Clarify this
	 * disable all BBUS Interrupts all but DMA controller but propagate
	 * later enabled ones to sys module 
	 */
	BBUS_IER = BBUS_IER_GLBL | BBUS_IER_USBE | BBUS_IER_DMAE;

	/* we need no handler for it. IRQ_BBUSAGG will trigger the BBUS
	 * handlers immediately in entry-armv.S */
	ns9xxx_unmask_ahb_irq( IRQ_BBUSAGG );
	ns9xxx_unmask_bbus_irq( IRQ_BBUS_DMA );

	/* setup board specific interrupt handlers */
	if (board_init_irq)
		board_init_irq();
}

