/*
 * arch/arm/mach-ns9xxx/processor-ns9360.c
 *
 * Copyright (C) 2006 by Digi International Inc.
 * All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License version 2 as published by
 * the Free Software Foundation.
 */

#include <linux/platform_device.h>

#include <asm/arch/regs-sys.h>
#include <asm/arch/regs-ser.h>
#include <asm/arch/processor.h>
#include <asm/arch/spi.h>

#include "processor-ns9360.h"

#if defined(CONFIG_SPI_NS9XXX) || defined(CONFIG_SPI_NS9XXX_MODULE)
static struct resource ns9xxx_spi_resource0[] = {
	[0] = {
		.start = SER_MODULE_BASE_PA,
		.end   = SER_MODULE_BASE_PA + 0x24,
		.flags = IORESOURCE_MEM,
	},
	[1] = {
		.start = IRQ_BBUS_DMA_1,
		.end   = IRQ_BBUS_DMA_2,
		.flags = IORESOURCE_IRQ,
	},
};
static struct resource ns9xxx_spi_resource1[] = {
	[0] = {
		.start = SER_MODULE_BASE_PA + SER_CH_OFF(1),
		.end   = SER_MODULE_BASE_PA + SER_CH_OFF(1) + 0x24,
		.flags = IORESOURCE_MEM,
	},
	[1] = {
		.start = IRQ_BBUS_DMA_3,
		.end   = IRQ_BBUS_DMA_4,
		.flags = IORESOURCE_IRQ,
	},
};
static struct resource ns9xxx_spi_resource2[] = {
	[0] = {
		.start = SER_MODULE_BASE_PA + SER_CH_OFF(2),
		.end   = SER_MODULE_BASE_PA + SER_CH_OFF(2) + 0x24,
		.flags = IORESOURCE_MEM,
	},
	[1] = {
		.start = IRQ_BBUS_DMA_5,
		.end   = IRQ_BBUS_DMA_6,
		.flags = IORESOURCE_IRQ,
	},
};
static struct resource ns9xxx_spi_resource3[] = {
	[0] = {
		.start = SER_MODULE_BASE_PA + SER_CH_OFF(3),
		.end   = SER_MODULE_BASE_PA + SER_CH_OFF(3) + 0x24,
		.flags = IORESOURCE_MEM,
	},
	[1] = {
		.start = IRQ_BBUS_DMA_7,
		.end   = IRQ_BBUS_DMA_8,
		.flags = IORESOURCE_IRQ,
	}
};


static u64 ns9xxx_spi_dmamask[4] = { 0xffffffffUL, 0xffffffffUL, 0xffffffffUL, 0xffffffffUL };
struct ns9xxx_spi_cfg spi_cfg[4] = {
	[0] = {
		.hwport             = SPI_PORT_0,	/* set to 1 to enable */
		.use_dma            = 1,
		.use_gpio_for_cs    = 0,
	},
	[1] = {	.hwport             = SPI_PORT_1, 
		.use_dma            = 1,
		.use_gpio_for_cs    = 0,
	},
	[2] = {	.hwport             = SPI_PORT_2, 
		.use_dma            = 1,
		.use_gpio_for_cs    = 0,
	},
	[3] = {	.hwport             = SPI_PORT_3, 
		.use_dma            = 1,
		.use_gpio_for_cs    = 0,
	},
};

struct platform_device ns9xxx_devices_spi[4] = {
	[0] = {
		.name		  = "ns9xxx_spi",
		.id		  = 0,
		.num_resources	  = ARRAY_SIZE(ns9xxx_spi_resource0),
		.resource	  = ns9xxx_spi_resource0,
		.dev              = {
			.platform_data = &spi_cfg[0],
			.dma_mask = &ns9xxx_spi_dmamask[0],
			.coherent_dma_mask = 0xffffffffUL
		}
	},
	[1] = {
		.name		  = "ns9xxx_spi",
		.id		  = 1,
		.num_resources	  = ARRAY_SIZE(ns9xxx_spi_resource1),
		.resource	  = ns9xxx_spi_resource1,
		.dev              = {
			.platform_data = &spi_cfg[1],
			.dma_mask = &ns9xxx_spi_dmamask[1],
			.coherent_dma_mask = 0xffffffffUL
		}
	},
	[2] = {
		.name		  = "ns9xxx_spi",
		.id		  = 2,
		.num_resources	  = ARRAY_SIZE(ns9xxx_spi_resource2),
		.resource	  = ns9xxx_spi_resource2,
		.dev              = {
			.platform_data = &spi_cfg[2],
			.dma_mask = &ns9xxx_spi_dmamask[2],
			.coherent_dma_mask = 0xffffffffUL
		}
	},
	[3] = {
		.name		  = "ns9xxx_spi",
		.id		  = 3,
		.num_resources	  = ARRAY_SIZE(ns9xxx_spi_resource3),
		.resource	  = ns9xxx_spi_resource3,
		.dev              = {
			.platform_data = &spi_cfg[3],
			.dma_mask = &ns9xxx_spi_dmamask[3],
			.coherent_dma_mask = 0xffffffffUL
		}
	},
};


void __init ns9xxx_register_spi_devices(void)
{
	int i;
	
	for (i = 0; i < 4; i++)	{
		if (spi_cfg[i].hwport) 
			platform_device_register(&ns9xxx_devices_spi[i]);
	}
}

#else
void __init ns9xxx_register_spi_devices(void) {}
#endif


void __init processor_ns9360_fixup(void)
{
	__processor_type = PROCESSOR_NS9360;
}
