/*
 * arch/arm/mach-ns9xxx/time.c
 *
 * Copyright (C) 2006 by Digi International Inc.
 * All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License version 2 as published by
 * the Free Software Foundation.
 */
#include <linux/version.h>

#include <linux/jiffies.h>
#include <linux/interrupt.h>
#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 18)
#include <asm/irq.h>
#else
#include <linux/irq.h>
#endif
#include <asm/arch-ns9xxx/regs-sys.h>
#include <asm/arch-ns9xxx/irqs.h>
#include <asm/arch/system.h>
#include "generic.h"

#define SYSTEMTIMER 0
#define TIMERCLOCKSELECT 64
#define TIMER_USECS_SH_NUM_LEFT		(4)
#define TIMER_USECS_SH_DEN_RIGHT	(12)
#define TIMER_USECS_SHIFT		(TIMER_USECS_SH_NUM_LEFT+TIMER_USECS_SH_DEN_RIGHT)


static u32 ticks_per_usec_shifted;

static irqreturn_t
ns9xxx_timer_interrupt(int irq, void *dev_id, struct pt_regs *regs)
{
	/* XXX: some platforms (e.g. sa1100, pxa) do some more things here
	 * to increase accurancy---so should I? */
	write_seqlock(&xtime_lock);
	timer_tick(regs);
	write_sequnlock(&xtime_lock);

	/* Acknowledge the timer interrupt */
	SYS_TC(SYSTEMTIMER) |= SYS_TCx_INTC;
	SYS_TC(SYSTEMTIMER) &= ~SYS_TCx_INTC;

	return IRQ_HANDLED;
}

static unsigned long ns9xxx_timer_gettimeoffset(void)
{
	/* XXX: some platforms (e.g. s3c2410) check if there is an timer
	 * interrupt pending */
	u32 elapsed = SYS_TRC(SYSTEMTIMER) - SYS_TR(SYSTEMTIMER);

	return ((elapsed << TIMER_USECS_SHIFT) / ticks_per_usec_shifted);
}

static struct irqaction ns9xxx_timer_irq = {
	.name = "NS9xxx Timer Tick",
#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 18)
	.flags = SA_INTERRUPT | SA_TIMER,
#else
	.flags = IRQF_DISABLED | IRQF_TIMER,
#endif
	.handler = ns9xxx_timer_interrupt,
};

static void __init ns9xxx_timer_init(void)
{
	int tc;

	*(u32 *)0xf9600024 |= 0x80000000;

	ticks_per_usec_shifted = (ns9xxx_cpuclock() << TIMER_USECS_SH_NUM_LEFT) /
				 ((TIMERCLOCKSELECT * 1000000) >> TIMER_USECS_SH_DEN_RIGHT);

	/* disable timer */
	if ((tc = SYS_TC(SYSTEMTIMER)) & SYS_TCx_TEN)
		SYS_TC(SYSTEMTIMER) = tc & ~SYS_TCx_TEN;

	SYS_TRC(SYSTEMTIMER) =
		SH_DIV(ns9xxx_cpuclock(), (TIMERCLOCKSELECT * HZ), 0);

	REGSET(tc, SYS_TCx, TEN, EN);
	REGSET(tc, SYS_TCx, TLCS, DIV64); /* TIMERCLOCKSELECT */
	REGSET(tc, SYS_TCx, INTS, EN);
	REGSET(tc, SYS_TCx, UDS, DOWN);
	REGSET(tc, SYS_TCx, TDBG, STOP);
	REGSET(tc, SYS_TCx, TSZ, 32);
	REGSET(tc, SYS_TCx, REN, EN);
	SYS_TC(SYSTEMTIMER) = tc;

	setup_irq(IRQ_TIMER0 + SYSTEMTIMER, &ns9xxx_timer_irq);
}

struct sys_timer ns9xxx_timer = {
	.init = ns9xxx_timer_init,
	.offset = ns9xxx_timer_gettimeoffset,
};
