/*
 * linux/arch/arm/mach-pxa/ccxp.c
 *
 * Support for the Digi ConnectCore XP Development Platform.
 *
 * @Author: Bernd Westermann
 * @References: [1] lubbock.c
 *
 * Copyright (C) 2004 by FS Forth-Systeme GmbH
 * All rights reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 *
 */

/***********************************************************************
 *  @History:
 *     2005/06/26 : initial Version
 ***********************************************************************/

#include <linux/module.h>
#include <linux/kernel.h>
#include <linux/init.h>
#include <linux/platform_device.h>
#include <linux/device.h>
#include <linux/sysdev.h>
#include <linux/major.h>
#include <linux/fb.h>
#include <linux/interrupt.h>

#include <asm/setup.h>
#include <asm/memory.h>
#include <asm/mach-types.h>
#include <asm/hardware.h>
#include <asm/irq.h>

#include <asm/mach/arch.h>
#include <asm/mach/map.h>
#include <asm/mach/irq.h>

#include <asm/hardware/sa1111.h>

#include <asm/arch/pxa-regs.h>
#include <asm/arch/ccxp.h>
#include <asm/arch/udc.h>
#include <asm/arch/pxafb.h>
#include <asm/arch/mmc.h>
#include <asm/arch/irda.h>
#include <linux/hdreg.h>
#include <linux/ide.h>
#include <linux/delay.h>

#include "generic.h"

static struct resource smc91x_resources[] = {
	[0] = {
		.name	= "smc91x-regs",
		.start	= CCXP_ETH1_PHYS /* + 0x300*/ /*0x0c000000*/,
		.end	= CCXP_ETH1_PHYS + 0xfffff /*0x0c0fffff*/,
		.flags	= IORESOURCE_MEM,
	},
	[1] = {
		.start	= CCXP_ETH_IRQ,
		.end	= CCXP_ETH_IRQ,
		.flags	= IORESOURCE_IRQ,
	},
};

static struct resource sl811_resources[] = {
	[0] = {
		.name	= "sl811-regs",
		.start	= CCXP_SL811_PHYS,
		.end	= CCXP_SL811_PHYS,
		.flags	= IORESOURCE_MEM,
	},
	[1] = {
		.name	= "sl811-data",
		.start	= CCXP_SL811_DATA,
		.end	= CCXP_SL811_DATA,
		.flags	= IORESOURCE_MEM,
	},
	[2] = {
		.name	= "sl811-irq",
		.start	= CCXP_SL811_IRQ,
		.end	= CCXP_SL811_IRQ,
		.flags	= IORESOURCE_IRQ,
	},
};

static struct resource pxa2xx_spi_ssp_resources[] = {
};

static struct platform_device pxa2xx_spi_ssp_device = {
	.name		= "pxa2xx-spi-ssp",
	.id		= -1,
	.num_resources	= ARRAY_SIZE(pxa2xx_spi_ssp_resources),
	.resource	= pxa2xx_spi_ssp_resources,
};

static struct platform_device smc91x_device = {
	.name		= "smc91x",
	.id		= -1,
	.num_resources	= ARRAY_SIZE(smc91x_resources),
	.resource	= smc91x_resources,
};

static struct platform_device sl811_device = {
	.name		= "sl811-hcd",
	.id		= -1,
	.num_resources	= ARRAY_SIZE(sl811_resources),
	.resource	= sl811_resources,
};

static struct pxafb_mach_info ccxp_lcd_data __initdata = {
	// use 640x480 as standard
	.pixclock =	40193,	/* clock period in ps => 24.88 MHz */
	.bpp =		16,
	.xres =		320,
	.yres =		240,
	.hsync_len =	64, // LCD_HORIZONTAL_SYNC_PULSE_WIDTH,
	.vsync_len =	3, // LCD_VERTICAL_SYNC_PULSE_WIDTH,
	.left_margin =	32, // LCD_BEGIN_OF_LINE_WAIT_COUNT,
	.upper_margin =	6, //32, // LCD_BEGIN_FRAME_WAIT_COUNT,
	.right_margin =	32, // LCD_END_OF_LINE_WAIT_COUNT,
	.lower_margin =	32, // LCD_END_OF_FRAME_WAIT_COUNT,
	.sync =		0, // HSYNC, VSYNC polarity
	.lccr0 =	LCCR0_PAS,
	.lccr3 =	LCCR3_PCP | LCCR3_Acb(0),
	.cmap_inverse = 0,
	.cmap_greyscale = 0,
};

#define CF_CD_MASK			0x0001
#define DRV_ADDR_SHIFT 2
#define DRV_ADDR_MASK (0x0f << DRV_ADDR_SHIFT)
static inline int CF_PRESENT(unsigned int base)
{
#if 0
	unsigned short ctrl_reg;

	// Detect if a CF is inserted on the socket
	ctrl_reg = readb( base + HD_CURRENT );
	if( ctrl_reg > 0 && ( ctrl_reg & CF_CD_MASK ) != CF_CD_MASK ) {
		printk("CF Card detected: %02x\n", ctrl_reg);
		return 1;
	}
	return 0;
#endif
	int present;
	unsigned char hd_cur;
	unsigned char drvaddr;
	/*
	* since bits 0..3 of HD_CURRENT are the invers of
	* bits 2..5 of HD_DRVADDR one of those will be != 0,
	* if a TRUE IDE CF card is inserted
	*/
	hd_cur = inb(base + CCXP_OFFSET_IDE_CS0 + HD_CURRENT);
	drvaddr = inb(base + CCXP_OFFSET_IDE_CS1 + HD_DRVADDR);

	present = ((hd_cur << DRV_ADDR_SHIFT) ^ drvaddr) & DRV_ADDR_MASK;

	return present == DRV_ADDR_MASK;
}

static struct resource ide_resources[] = {
	{
		.name	= "ide-mem0",
		.start	= CCXP_PHYS_IDE1_BASE + CCXP_OFFSET_IDE_CS0,
		.end	= CCXP_PHYS_IDE1_BASE + CCXP_OFFSET_IDE_CS0 + 7,
		.flags	= IORESOURCE_MEM,
	},
	{
		.name	= "ide-mem1",
		.start	= CCXP_PHYS_IDE1_BASE + CCXP_OFFSET_IDE_CS1 + HD_CMD,
		.end	= CCXP_PHYS_IDE1_BASE + CCXP_OFFSET_IDE_CS1 + HD_CMD,
		.flags	= IORESOURCE_MEM,
	},
	{
		.name	= "ide-irq",
		.start	= IRQ_HARDDISK,
		.end	= IRQ_HARDDISK,
		.flags	= IORESOURCE_IRQ,
	},
};
	
static struct platform_device ide_device = {
	.name		= "ccxp-ide",
	.id		= 0,
	.num_resources	= ARRAY_SIZE(ide_resources),
	.resource	= ide_resources,
};

#ifdef CONFIG_IDE_MODULE
#define __ide_init
#else
#define __ide_init __init
#endif
int __ide_init ccxp_ide_init(void *data)
{
	hw_regs_t *hw = data;
	struct platform_device *pdev = &ide_device;
	struct resource *res;
	unsigned long base1, base2;
	int retries = 10;
	int cffound=0;

#if 0 /*MOBIPX PNDPND*/
	return -ENODEV;
#endif

#if 1 /* TEST for 2.6.15-rc5 */
	return -ENODEV;
#endif
	printk("%s: Initializing IDE interface\n", __FUNCTION__);

	mdelay(400);
	while (retries > 0) {
		mdelay(40);
		if (CF_PRESENT(CCXP_IDE0_BASE)) {
			printk("%s: TRUE IDE CF card found @ %08lx\n",
			       __FUNCTION__, CCXP_IDE0_BASE);
			retries = 0;
			cffound=1;
		} else if (CF_PRESENT(CCXP_IDE1_BASE)) {
			printk("%s: TRUE IDE CF card found @ %08lx\n",
			       __FUNCTION__, CCXP_IDE1_BASE);
			retries = 0;
			cffound=1;
		} else {
			retries--;
		}
	}
	if (cffound==0) return  -ENODEV;

 	pxa_gpio_mode(GPIO80_nCS_4 | GPIO_IN);
 	pxa_gpio_mode(GPIO33_nCS_5 | GPIO_IN);

	hw->irq = platform_get_irq(pdev, 0);
	if (hw->irq != NO_IRQ) {
		set_irq_type(hw->irq, IRQT_RISING);
	}

	res = platform_get_resource(pdev, IORESOURCE_MEM, 0);
	if (!res) {
		printk("%s: Failed to get IOMEM 0 resource\n", __FUNCTION__);
		return -ENODEV;
	}
	base1 = (unsigned long)ioremap(res->start, res->end - res->start + 1);
	if (!base1) {
		printk("%s: Failed to ioremap %08lx..%08lx\n", __FUNCTION__,
			 res->start, res->end);
		return -ENOMEM;
	}

	res = platform_get_resource(pdev, IORESOURCE_MEM, 1);
	if (!res) {
		printk("%s: Failed to get IOMEM 1 resource\n", __FUNCTION__);
		return -ENODEV;
	}
	base2 = (unsigned long)ioremap(res->start, res->end - res->start + 1);
	if (!base2) {
		printk("%s: Failed to ioremap %08lx..%08lx\n", __FUNCTION__,
			 res->start, res->end);
		return -ENOMEM;
	}

	printk("%s: Initializing IDE HWIF %08lx:%08lx, IRQ %d\n", __FUNCTION__,
		 base1, base2, hw->irq);
	ide_std_init_ports(hw, base1, base2);

	return 0;
}
EXPORT_SYMBOL(ccxp_ide_init);

static struct platform_device *devices[] __initdata = {
	&smc91x_device,
	&sl811_device,
/*	&sl811_device, MOBIPX PNDPND*/
	&ide_device,
	&pxa2xx_spi_ssp_device, /*muss dass??????*/
};

static void ccxp_irda_transceiver_mode(struct device *dev, int mode)
{
}

static struct pxaficp_platform_data ccxp_ficp_platform_data = {
	.transceiver_cap  = IR_SIRMODE | IR_FIRMODE | IR_OFF,
	.transceiver_mode = ccxp_irda_transceiver_mode,
};

static void __init ccxp_init(void)
{
	pxa_set_ficp_info(&ccxp_ficp_platform_data);
	set_pxa_fb_info(&ccxp_lcd_data);
	(void) platform_add_devices(devices, ARRAY_SIZE(devices));
}

static void __init ccxp_init_irq(void)
{
	pxa_init_irq();
	set_irq_type(IRQ_GPIO1, IRQT_BOTHEDGE);
}

static struct map_desc ccxp_io_desc[] __initdata = {
/* PNDPND test it if all OK???	{ CCXP_ETH1_BASE, CCXP_ETH1_PHYS, CCXP_ETH1_SIZE,	MT_DEVICE },
	{ CCXP_SL811_BASE, CCXP_SL811_PHYS, CCXP_SL811_SIZE,	MT_DEVICE },
	{ CCXP_PCMCIA_IO_0_BASE,	CCXP_PCMCIA_PHYS_IO_0_BASE,	CCXP_PCMCIA_IO_0_MAPSIZE,	MT_DEVICE },
	{ CCXP_PCMCIA_IO_1_BASE,	CCXP_PCMCIA_PHYS_IO_1_BASE,	CCXP_PCMCIA_IO_1_MAPSIZE,	MT_DEVICE },*/
};

static void __init ccxp_map_io(void)
{
	pxa_map_io();
	iotable_init(ccxp_io_desc, ARRAY_SIZE(ccxp_io_desc));

	// PCMCIA / USB 
	/*
	 * Setup default state of GPIO outputs
	 * before we enable them as outputs.
	 */
 	GPSR(GPIO48_nPOE) =
 		GPIO_bit(GPIO48_nPOE) |
 		GPIO_bit(GPIO49_nPWE) |
 		GPIO_bit(GPIO50_nPIOR) |
 		GPIO_bit(GPIO51_nPIOW) |
 		GPIO_bit(GPIO52_nPCE_1) |
 		GPIO_bit(GPIO53_nPCE_2);
 
 	pxa_gpio_mode(GPIO48_nPOE_MD);
 	pxa_gpio_mode(GPIO49_nPWE_MD);
 	pxa_gpio_mode(GPIO50_nPIOR_MD);
 	pxa_gpio_mode(GPIO51_nPIOW_MD);
 	pxa_gpio_mode(GPIO52_nPCE_1_MD);
/* 	pxa_gpio_mode(GPIO53_nPCE_2_MD); */

 	pxa_gpio_mode(GPIO104_pSKTSEL_MD);
 	pxa_gpio_mode(GPIO55_nPREG_MD);
 	pxa_gpio_mode(GPIO56_nPWAIT_MD);
 	pxa_gpio_mode(GPIO57_nIOIS16_MD);
	pxa_gpio_mode(GPIO18_RDY_MD);

	// LCD GPIO
	pxa_gpio_mode(GPIO58_LDD_0_MD);
	pxa_gpio_mode(GPIO59_LDD_1_MD);
	pxa_gpio_mode(GPIO60_LDD_2_MD);
	pxa_gpio_mode(GPIO61_LDD_3_MD);
	pxa_gpio_mode(GPIO62_LDD_4_MD);
	pxa_gpio_mode(GPIO63_LDD_5_MD);
	pxa_gpio_mode(GPIO64_LDD_6_MD);
	pxa_gpio_mode(GPIO65_LDD_7_MD);
	pxa_gpio_mode(GPIO66_LDD_8_MD);
	pxa_gpio_mode(GPIO67_LDD_9_MD);
	pxa_gpio_mode(GPIO68_LDD_10_MD);
	pxa_gpio_mode(GPIO69_LDD_11_MD);
	pxa_gpio_mode(GPIO70_LDD_12_MD);
	pxa_gpio_mode(GPIO71_LDD_13_MD);
	pxa_gpio_mode(GPIO72_LDD_14_MD);
	pxa_gpio_mode(GPIO73_LDD_15_MD);
	pxa_gpio_mode(GPIO74_LCD_FCLK_MD);
	pxa_gpio_mode(GPIO75_LCD_LCLK_MD);
	pxa_gpio_mode(GPIO76_LCD_PCLK_MD);
	pxa_gpio_mode(GPIO77_LCD_ACBIAS_MD);

/*	//DS1339
	gpio_set_bit(CCXP_RTC_SDA_GPIO);
	pxa_gpio_mode(CCXP_RTC_SDA_GPIO | GPIO_OUT);
	gpio_set_bit(CCXP_RTC_SCL_GPIO);
	pxa_gpio_mode(CCXP_RTC_SCL_GPIO | GPIO_OUT);*/
}

MACHINE_START(CCXP, "Digi ConnectCore XP 270 Development Platform (Intel PXA270)")
	.phys_ram	= PXA_PHYS_RAM,
	.phys_io	= PXA_PHYS_IO_BASE,
	.io_pg_offst	= (io_p2v(PXA_PHYS_IO_BASE) >> 18) & 0xfffc,
	.map_io		= ccxp_map_io,
	.init_irq	= ccxp_init_irq,
	.timer		= &pxa_timer,
	.init_machine	= ccxp_init,
	.boot_params	= PXA_BOOT_PARAMS,
MACHINE_END
