/*
 * SPI support for pnx010x.
 *
 * Author: Dennis Kovalev <dkovalev@ru.mvista.com>
 *
 * 2004 (c) MontaVista Software, Inc. This file is licensed under
 * the terms of the GNU General Public License version 2. This program
 * is licensed "as is" without any warranty of any kind, whether express
 * or implied.
 */

#ifndef _SPI_PNX_BUS_DRIVER
#define _SPI_PNX_BUS_DRIVER

#include <asm/arch/platform.h>
#include <asm/arch/dma.h>
#include <asm/dma.h>
#include <asm/arch/spi.h>

/* structures */
struct spipnx_driver_data {
	unsigned io_flags;

	int irq;

	int dma_mode;
	int dma_channel;
	int slave_nr;

	u32 iostart, ioend;
	vhblas_spiregs *spi_regs;

	struct clk *clk;
	int clk_id[4];

	int state;
	u32 saved_clock;

	struct completion op_complete;
	struct completion threshold;

	int progress;

	void (*control) (struct spi_device * dev, int code, u32 ctl);
};

#define SPIPNX_STATE_UNINITIALIZED 0
#define SPIPNX_STATE_READY         1
#define SPIPNX_STATE_SUSPENDED     2

#define SPIPNX_IS_READY( bus )  ( (bus)->state == SPIPNX_STATE_READY )

#define FIFO_CHUNK_SIZE		56

#define SPI_ENDIAN_SWAP_NO                 0
#define SPI_ENDIAN_SWAP_YES                1

#endif				// __SPI_PNX_BUS_DRIVER
