/*
 *  linux/include/asm-arm/arch-ns9750/a9m9xx0_ts.h
 *
 * Copyright (C) 2005 by Sistemas Embebidos S.A.
 * A Digi International Company.
 * All rights reserved.
 *
 * @Author: Pedro Perez de Heredia <pperez@embebidos.com>
 * @Desc: Touch Screen header file for the touch interface on the NS9750/
 *        NS9360 processors. The touchscreen interface is implemented using
 *        one SPI channel of the processor. To the SPI bus, an ADS7843
 *        controller is connected.
 * @References: [1] NS9360 Hardware Reference Manual Version 90000675_A
 *                  March 2005.
 *              [2] ADS7843 Data Sheet.
 * @TODO:
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef __ASM_ARCH_A9M9XX0DEV_TS_H
#define __ASM_ARCH_A9M9XX0DEV_TS_H

/* ADS7843 control byte bit definitions */
#define ADS7843_CTRL_PD0	(0x01 << 0)   /* PD0 */
#define ADS7843_CTRL_PD1	(0x01 << 1)   /* PD1 */
#define ADS7843_CTRL_DFR	(0x01 << 2)   /* SER/DFR */
#define ADS7843_CTRL_MODE	(0x01 << 3)   /* MODE */
#define ADS7843_CTRL_ADDR_MA	0x70          /* Address mask */
#define ADS7843_CTRL_ADDR_SH	4             /* Address num bits to shift */
#define ADS7843_CTRL_S		(0x01 << 7)   /* Start Bit */

/* ADS7843 masks and max/min values */
#define ADS7843_XY_MASK			0xfff
#define ADS7843_Y_MASK		0xfff
#define ADS7843_X_MAX		0xfff
#define ADS7843_Y_MAX		0xfff
#define ADS7843_X_MIN		0
#define ADS7843_Y_MIN		0

/* Addresses */
#define ADS7843_GET_X_ADDR	((0x01 << ADS7843_CTRL_ADDR_SH) & ADS7843_CTRL_ADDR_MA)
#define ADS7843_GET_Y_ADDR	((0x05 << ADS7843_CTRL_ADDR_SH) & ADS7843_CTRL_ADDR_MA)

/* Commands used */
#define GET_ABS_X		((ADS7843_CTRL_S|ADS7843_GET_X_ADDR) << 8)
#define GET_ABS_Y		((ADS7843_CTRL_S|ADS7843_GET_Y_ADDR) << 8)


struct a9m9xx0_ts_data {
	int	samples_ms;	// Number of miliseconds between samples
	int	samples_avg;	// Number of samples to average
	int	port;		// Serial port used (0-PortA, 1-PortB...)
	int	bitrate;	// SPI bitrate in bps

	int	y_top;
	int	y_bottom;
	int	x_right;
	int	x_left;
};

void __init set_a9m9xx0ts_data( struct a9m9xx0_ts_data *hard_a9m9xx0ts_data );

#endif /* __ASM_ARCH_A9M9XX0DEV_TS_H */

