/*
 *  linux/include/asm-arm/arch-ns9750/a9m9xx0dev.h
 *
 * Copyright (C) 2005 by Sistemas Embebidos S.A.
 * A Digi International Company.
 * All rights reserved.
 *
 * @Author: Pedro Perez de Heredia <pperez@embebidos.com>
 * @References: [1] Logic Specification for A9M9XX0DEV_X V2.0
 *                  May 2005.
 * @TODO:
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef __ASM_ARCH_A9M9XX0DEV_H
#define __ASM_ARCH_A9M9XX0DEV_H


#define NS9XX0_CS0_PHYS		(0x40000000)
#define NS9XX0_CS1_PHYS		(0x50000000)
#define NS9XX0_CS2_PHYS		(0x60000000)
#define NS9XX0_CS3_PHYS		(0x70000000)

#define NS9XX0_FPGA_8B_PHYS	NS9XX0_CS0_PHYS
#define NS9XX0_FPGA_16B_PHYS	NS9XX0_CS2_PHYS

#define NS9XX0_FPGA_8B_VIRT	(0xf4000000)
#define NS9XX0_FPGA_16B_VIRT	(0xf6000000)

#define NS_FPGA_8B_P2V(x)	((x) - NS9XX0_FPGA_8B_PHYS + NS9XX0_FPGA_8B_VIRT)
#define NS_FPGA_16B_P2V(x)	((x) - NS9XX0_FPGA_16B_PHYS + NS9XX0_FPGA_16B_VIRT)
#define NS_FPGA_8B_V2P(x)	((x) - NS9XX0_FPGA_8B_VIRT + NS9XX0_FPGA_8B_PHYS)
#define NS_FPGA_16B_V2P(x)	((x) - NS9XX0_FPGA_16B_VIRT + NS9XX0_FPGA_16B_PHYS)

//#ifndef __ASSEMBLY__
//# define __FPGA_REG8(x)		(*((volatile unsigned char *)NS_FPGA_8B_P2V(x)))
//# define __FPGA_REG16(x)	(*((volatile unsigned short *)NS_FPGA_16B_P2V(x)))
//#else
# define __FPGA_REG8(x)		NS_FPGA_8B_P2V(x)
# define __FPGA_REG16(x)	NS_FPGA_16B_P2V(x)
//#endif


/* FPGA internal registers virtual addresses */
#define BASE_EXT_UARTA		__FPGA_REG8(NS9XX0_FPGA_8B_PHYS + 0x0000)
#define BASE_EXT_UARTB		__FPGA_REG8(NS9XX0_FPGA_8B_PHYS + 0x0008)
#define BASE_EXT_UARTC		__FPGA_REG8(NS9XX0_FPGA_8B_PHYS + 0x0010)
#define BASE_EXT_UARTD		__FPGA_REG8(NS9XX0_FPGA_8B_PHYS + 0x0018)
#define FPGA_ACR		__FPGA_REG8(NS9XX0_FPGA_8B_PHYS + 0x0030)
#define FPGA_LVR		__FPGA_REG8(NS9XX0_FPGA_8B_PHYS + 0x0040)
#define FPGA_IER		__FPGA_REG8(NS9XX0_FPGA_8B_PHYS + 0x0050)
#define FPGA_ISR		__FPGA_REG8(NS9XX0_FPGA_8B_PHYS + 0x0060)

#define FPGA_BVR		__FPGA_REG16(NS9XX0_FPGA_16B_PHYS + 0x0000)
#define FPGA_LVR2		__FPGA_REG16(NS9XX0_FPGA_16B_PHYS + 0x0010)
#define FPGA_CAN0_BASE		__FPGA_REG16(NS9XX0_FPGA_16B_PHYS + 0x1000)
#define FPGA_CAN1_BASE		__FPGA_REG16(NS9XX0_FPGA_16B_PHYS + 0x2000)

#define FPGA_CF_CONF		__FPGA_REG16(NS9XX0_FPGA_16B_PHYS + 0x10000)
#define FPGA_CF_IO8		__FPGA_REG16(NS9XX0_FPGA_16B_PHYS + 0x20000)
#define FPGA_CF_IO16		__FPGA_REG16(NS9XX0_FPGA_16B_PHYS + 0x50000)
#define FPGA_CF_ATR		__FPGA_REG16(NS9XX0_FPGA_16B_PHYS + 0x80000)
#define FPGA_CF_CM		__FPGA_REG16(NS9XX0_FPGA_16B_PHYS + 0xb0000)
#define FPGA_CF_SIZE		(0xe0000 - 0x10000)

/* FPGA ISR masks */
#define IRQ_FPGA_UARTA_MA	(0x01 << 0)
#define IRQ_FPGA_UARTB_MA	(0x01 << 1)
#define IRQ_FPGA_UARTC_MA	(0x01 << 2)
#define IRQ_FPGA_UARTD_MA	(0x01 << 3)
#define IRQ_FPGA_TOUCH_MA	(0x01 << 4)
#define IRQ_FPGA_CF_MA		(0x01 << 5)
#define IRQ_FPGA_CAN0_MA	(0x01 << 6)
#define IRQ_FPGA_CAN1_MA	(0x01 << 7)

#endif //__ASM_ARCH_A9M9XX0DEV_H

