/*
 * include/asm-arm/arch-ns9xxx/irqs.h
 *
 * Copyright (C) 2006 by Digi International Inc.
 * All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License version 2 as published by
 * the Free Software Foundation.
 */
#ifndef __ASM_ARCH_IRQS_H
#define __ASM_ARCH_IRQS_H

#define AHB_IRQ(irq)		(0 + irq)

#define IRQ_WATCHDOG		AHB_IRQ(0)
#define IRQ_AHBBUSERR		AHB_IRQ(1)
#define IRQ_BBUSAGG		AHB_IRQ(2)
/* irq 3 is reserved for NS9360 */
#define IRQ_ETHRX		AHB_IRQ(4)
#define IRQ_ETHTX		AHB_IRQ(5)
#define IRQ_ETHPHY		AHB_IRQ(6)
#define IRQ_LCD			AHB_IRQ(7)
#if defined(CONFIG_PROCESSOR_NS9360)
# define IRQ_SERBRX		AHB_IRQ(8)
# define IRQ_SERBTX		AHB_IRQ(9)
# define IRQ_SERARX		AHB_IRQ(10)
# define IRQ_SERATX		AHB_IRQ(11)
# define IRQ_SERCRX		AHB_IRQ(12)
# define IRQ_SERCTX		AHB_IRQ(13)
#elif defined(CONFIG_PROCESSOR_NS9750)
# define IRQ_PCI_BRIDGE		AHB_IRQ(8)
# define IRQ_PCI_ARBITER	AHB_IRQ(9)
# define IRQ_PCI_EXT_0		AHB_IRQ(10)
# define IRQ_PCI_EXT_1		AHB_IRQ(11)
# define IRQ_PCI_EXT_2		AHB_IRQ(12)
# define IRQ_PCI_EXT_3		AHB_IRQ(13)
#endif
#define IRQ_I2C			AHB_IRQ(14)
#define IRQ_BBUSDMA		AHB_IRQ(15)
#define IRQ_TIMER0		AHB_IRQ(16)
#define IRQ_TIMER1		AHB_IRQ(17)
#define IRQ_TIMER2		AHB_IRQ(18)
#define IRQ_TIMER3		AHB_IRQ(19)
#define IRQ_TIMER4		AHB_IRQ(20)
#define IRQ_TIMER5		AHB_IRQ(21)
#define IRQ_TIMER6		AHB_IRQ(22)
#define IRQ_TIMER7		AHB_IRQ(23)
#define IRQ_RTC			AHB_IRQ(24)
#define IRQ_USBHOST		AHB_IRQ(25)
#define IRQ_USBDEVICE		AHB_IRQ(26)
#define IRQ_IEEE1284		AHB_IRQ(27)
#define IRQ_EXT0		AHB_IRQ(28)
#define IRQ_EXT1		AHB_IRQ(29)
#define IRQ_EXT2		AHB_IRQ(30)
#define IRQ_EXT3		AHB_IRQ(31)
#define MAX_AHB_IRQ		AHB_IRQ(31)

#define BBUS_IRQ(irq)		(MAX_AHB_IRQ + irq + 1)

#define IRQ_BBUS_DMA		BBUS_IRQ(0)
#define IRQ_BBUS_USB		BBUS_IRQ(1)
#define IRQ_BBUS_SERBRX		BBUS_IRQ(2)
#define IRQ_BBUS_SERBTX		BBUS_IRQ(3)
#define IRQ_BBUS_SERARX		BBUS_IRQ(4)
#define IRQ_BBUS_SERATX		BBUS_IRQ(5)
#define IRQ_BBUS_SERCRX		BBUS_IRQ(6)
#define IRQ_BBUS_SERCTX		BBUS_IRQ(7)
#define IRQ_BBUS_SERDRX		BBUS_IRQ(8)
#define IRQ_BBUS_SERDTX		BBUS_IRQ(9)
#define IRQ_BBUS_I2C		BBUS_IRQ(10)
#define IRQ_BBUS_1284		BBUS_IRQ(11)
#define IRQ_BBUS_UTIL		BBUS_IRQ(12)
#define IRQ_BBUS_RTC		BBUS_IRQ(13)
#define IRQ_BBUS_USBHST		BBUS_IRQ(14)
#define IRQ_BBUS_USBDEV		BBUS_IRQ(15)
#define IRQ_BBUS_AHBDMA1	BBUS_IRQ(24)
#define IRQ_BBUS_AHBDMA2	BBUS_IRQ(25)
#define MAX_BBUS_IRQ		BBUS_IRQ(25)

#define BBUS_DMA_IRQ(irq)	(MAX_BBUS_IRQ + irq + 1)

#define IRQ_BBUS_DMA_1		BBUS_DMA_IRQ(0)
#define IRQ_BBUS_DMA_2		BBUS_DMA_IRQ(1)
#define IRQ_BBUS_DMA_3		BBUS_DMA_IRQ(2)
#define IRQ_BBUS_DMA_4		BBUS_DMA_IRQ(3)
#define IRQ_BBUS_DMA_5		BBUS_DMA_IRQ(4)
#define IRQ_BBUS_DMA_6		BBUS_DMA_IRQ(5)
#define IRQ_BBUS_DMA_7		BBUS_DMA_IRQ(6)
#define IRQ_BBUS_DMA_8		BBUS_DMA_IRQ(7)
#define IRQ_BBUS_DMA_9		BBUS_DMA_IRQ(8)
#define IRQ_BBUS_DMA_10		BBUS_DMA_IRQ(9)
#define IRQ_BBUS_DMA_11		BBUS_DMA_IRQ(10)
#define IRQ_BBUS_DMA_12		BBUS_DMA_IRQ(11)
#define IRQ_BBUS_DMA_13		BBUS_DMA_IRQ(12)
#define IRQ_BBUS_DMA_14		BBUS_DMA_IRQ(13)
#define IRQ_BBUS_DMA_15		BBUS_DMA_IRQ(14)
#define IRQ_BBUS_DMA_16		BBUS_DMA_IRQ(15)
#define MAX_BBUS_DMA_IRQ	BBUS_DMA_IRQ(15)

#define USB_IRQ(irq)		(MAX_BBUS_DMA_IRQ + irq + 1)

#define IRQ_USB_DMA		USB_IRQ(0)
#define IRQ_USB_HOST		USB_IRQ(1)
#define IRQ_USB_FIFO		USB_IRQ(2)
#define MAX_USB_IRQ		USB_IRQ(2)

#include <asm/arch/board.h>	/* will need some rework*/

#ifndef LAST_CPU_IRQ
# define LAST_CPU_IRQ		MAX_USB_IRQ
#endif

#ifndef	NR_IRQS
# define NR_IRQS		(LAST_CPU_IRQ + 1)
#endif

#endif /* __ASM_ARCH_IRQS_H */
