/*
 * include/asm-arm/arch-ns9xxx/regs-bbus.h
 *
 * Copyright (C) 2006 by Digi International Inc.
 * All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License version 2 as published by
 * the Free Software Foundation.
 */

#ifndef __ASM_ARCH_REGSBBUS_H
#define __ASM_ARCH_REGSBBUS_H

#include <asm/arch/hardware.h>

/* BBus Utility registers and addresses */
#define BBUS_UTILITY_BASE_PA		(0x90600000)
#define BBUS_UTILITY_BASE_VA		io_p2v(BBUS_UTILITY_BASE_PA)

#define BBUS_MODULE_BASE_PA		(0x90000000)
#define BBUS_MODULE_BASE_VA		io_p2v(BBUS_MODULE_BASE_PA)


/* Master Reset Register */
#define BBUS_MRST			__REG(0x90600000)

/* BBUS Utility Interrupt Status Register */
#define BBUS_UISR			__REG(0x90600004)

/* GPIO configuration registers and addresses */
#define NS9XXX_CFG_ADDR_BASE_VA		(BBUS_UTILITY_BASE_VA + 0x10)
#define NS9XXX_CFG_PIN_OFFSET(pin)	((((pin) >> 3) *4) +(((pin) <56)?0:212))

#define NS9XXX_CTRL_ADDR_BASE_VA	(BBUS_UTILITY_BASE_VA + 0x30)
#define NS9XXX_CTRL_PIN_OFFSET(pin)	((((pin) >> 5) *4) +(((pin) <64)?0:232))

#define NS9XXX_STAT_ADDR_BASE_VA	(BBUS_UTILITY_BASE_VA + 0x40)	
#define NS9XXX_STAT_PIN_OFFSET(pin)	((((pin) >> 5) *4) +(((pin) <64)?0:232))	

/* BBUS Time Out Register */
#define BBUS_TOUT			__REG(0x90600050)


/* BBUS DMA Interrupt Status Register */
#define BBUS_DMA_ISTAT			__REG(0x90600060)

/* BBUS DMA Interrupt Enable Register */
#define BBUS_DMA_IEN			__REG(0x90600064)

/* BBUS USB Configuration Register */
#define BBUS_USB_CFG			__REG(0x90600070)

/* Endian Configuration Register */
#define BBUS_END_CFG			__REG(0x90600080)

/* ARM Wake-up Register */
#define BBUS_WAKEUP			__REG(0x90600090)


#define BBUS_DMA_DESCR_VA(ctrl,ch)	(BBUS_MODULE_BASE_VA + \
					 (ctrl) * 0x110000 + (ch) * 0x20)
#define BBUS_DMA_CTRL_VA(ctrl,ch)	(BBUS_MODULE_BASE_VA + \
					 (ctrl) * 0x110000 + (ch) * 0x20+0x10)
#define BBUS_DMA_STAT_VA(ctrl,ch)	(BBUS_MODULE_BASE_VA + \
					 (ctrl) * 0x110000 + (ch) * 0x20 + 0x14)


/* register bit fields */
#define NS_BBUS_USB_CFG_OUTEN		(0x00000008)
#define NS_BBUS_USB_CFG_SPEED		(0x00000004)
#define NS_BBUS_USB_CFG_CFG_MA		(0x00000003)
#define NS_BBUS_USB_CFG_CFG_HOST_SOFT 	(0x00000003)
#define NS_BBUS_USB_CFG_CFG_DEVICE	(0x00000002)
#define NS_BBUS_USB_CFG_CFG_HOST	(0x00000001)
#define NS_BBUS_USB_CFG_CFG_DIS		(0x00000000)

#ifdef CONFIG_PROCESSOR_NS9360
#define NS_BBUS_USB_CFG_EXT_PHY_UNI_DIR	(0x00000020)
#define NS_BBUS_USB_CFG_INT_PHY_USBD	(0x00000010)
#define NS_BBUS_USB_CFG_SPEED		(0x00000004)
#endif

#define NS_BBUS_ENDIAN_CFG_AHBM		(0x00001000)
#define NS_BBUS_ENDIAN_CFG_I2C		(0x00000080)
#define NS_BBUS_ENDIAN_CFG_IEEE1284	(0x00000040)
#define NS_BBUS_ENDIAN_CFG_SER4		(0x00000020)
#define NS_BBUS_ENDIAN_CFG_SER3		(0x00000010)
#define NS_BBUS_ENDIAN_CFG_SER2		(0x00000008)
#define NS_BBUS_ENDIAN_CFG_SER1		(0x00000004)
#define NS_BBUS_ENDIAN_CFG_USB		(0x00000002)
#define NS_BBUS_ENDIAN_CFG_DMA		(0x00000001)



/* XXX modify to use __REGBIT macro */
#define GPIO_CFG_FUNC_0			(0x00)
#define GPIO_CFG_FUNC_1			(0x01)
#define GPIO_CFG_FUNC_2			(0x02)
#define GPIO_CFG_INPUT			(0x03)
#define GPIO_CFG_OUTPUT			(0x0b)
#define GPIO_CFG_INVERT			(0x04)


#endif /* ifndef __ASM_ARCH_REGSBBUS_H */


