/*
 * include/asm-arm/arch-ns9xxx/regs-lcd.h
 *
 * Copyright (C) 2006 by Digi International Inc.
 * All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License version 2 as published by
 * the Free Software Foundation.
 */
#ifndef __ASM_ARCH_REGSLCD_H
#define __ASM_ARCH_REGSLCD_H

#include <asm/arch/hardware.h>

/* LCD Controller */

/* Serial Module control module UART, register base addresses and offsets */
#define LCD_MODULE_BASE_PA	(0xa0800000)
#define LCD_MODULE_BASE_VA	io_p2v(LCD_MODULE_BASE_PA)


/* LCDTiming0, Horizontal axis panel control */
#define LCD_TIMING0		__REG(0xa0800000)

/* LCDTiming1, Vertical axis panel control */
#define LCD_TIMING1		__REG(0xa0800004)

/* LCDTiming2, Clock and signal polarity control */
#define LCD_TIMING2		__REG(0xa0800008)

/* LCDTiming3, Line end control */
#define LCD_TIMING3		__REG(0xa080000c)

/* For sequential access */
#define LCD_TIMINGX(x)		__REG2(0xa0800000,(x))

/* LCDUPBASE, Upper panel frame base address */
#define LCD_UPBASE		__REG(0xa0800010)

/* LCDLPBASE, Lower panel frame base address */
#define LCD_LPBASE		__REG(0xa0800014)

/* LCDINTRENABLE, Interrupt enable mask */
#define LCD_LCDINTRENABLE	__REG(0xa0800018)

/* LCDControl, LCD panel pixel parameters */
#define LCD_CONTROL		__REG(0xa080001c)

/* LCDStatus, Raw interrupt status */
#define LCD_STATUS		__REG(0xa0800020)

/* LCDInterrupt, Final masked interrupts */
#define LCD_INTERRUPT		__REG(0xa0800024)

/* LCDUPCURR, LCD upper panel current address value */
#define LCD_UPCURRENT		__REG(0xa0800028)

/* LCDLPCURR, LCD lower panel current address value */
#define LCD_LPCURRENT		__REG(0xa080002c)

/* LCDPalette, 256x16 bit color palette */
#define LCD_PALETTE(x)		__REG2(0xa0800200,x)


/* XXX Adapt to use hardware.h macros */
/* Register bit fields */
#define LCD_TIMING_0_HBP(x)		(((x)&0xFF) << 24)
#define LCD_TIMING_0_HFP(x)		(((x)&0xFF) << 16)
#define LCD_TIMING_0_HSW(x)		(((x)&0xFF) <<  8)
#define LCD_TIMING_0_PPL(x)		(((x)&0x3F) <<  2)

#define LCD_TIMING_1_VBP(x)		(((x)&0xFF) << 24)
#define LCD_TIMING_1_VFP(x)		(((x)&0xFF) << 16)
#define LCD_TIMING_1_VSW(x)		(((x)&0x3F) << 10)
#define LCD_TIMING_1_LPP(x)		(((x)&0x3FF))

#define LCD_TIMING_2_BCD		(0x04000000)
#define LCD_TIMING_2_CPL(x)		(((x)&0x3FF) << 16)
#define LCD_TIMING_2_IOE		(0x00004000)
#define LCD_TIMING_2_IPC		(0x00002000)
#define LCD_TIMING_2_IHS		(0x00001000)
#define LCD_TIMING_2_IVS		(0x00000800)
#define LCD_TIMING_2_ACB(x)		(((x)&0x1F) << 6)
#define LCD_TIMING_2_PCD(x)		(((x)&0x1F))

#define LCD_TIMING_3_LEE		(0x00010000)
#define LCD_TIMING_3_LED(x)		(((x)&0x7F))

#define LCD_UPBASE_V(x)			(((x)&0x3FFFFFFF)<<2)

#define LCD_PBASE_V(x)			(((x)&0x3FFFFFFF)<<2)

#define LCD_INTREN_MBERR		(0x00000008)
#define LCD_INTREN_VCOMP		(0x00000004)
#define LCD_INTREN_LNBU			(0x00000002)

#define LCD_CONTROL_WATERMARK		(0x00010000)
#define LCD_CONTROL_VCOMP_MA		(0x00003000)
#define LCD_CONTROL_VCOMP_VS		(0x00000000)
#define LCD_CONTROL_VCOMP_BP		(0x00001000)
#define LCD_CONTROL_VCOMP_AV		(0x00002000)
#define LCD_CONTROL_VCOMP_FP		(0x00003000)
#define LCD_CONTROL_PWR			(0x00000800)
#define LCD_CONTROL_BEPO		(0x00000400)
#define LCD_CONTROL_BEBO		(0x00000200)
#define LCD_CONTROL_BGR			(0x00000100)
#define LCD_CONTROL_DUAL		(0x00000080)
#define LCD_CONTROL_MONO8		(0x00000040)
#define LCD_CONTROL_TFT			(0x00000020)
#define LCD_CONTROL_BW			(0x00000010)
#define LCD_CONTROL_BPP_MA		(0x0000000E)
#define LCD_CONTROL_BPP_1 		(0x00000000)
#define LCD_CONTROL_BPP_2 		(0x00000002)
#define LCD_CONTROL_BPP_4 		(0x00000004)
#define LCD_CONTROL_BPP_8 		(0x00000006)
#define LCD_CONTROL_BPP_16 		(0x00000008)
#define LCD_CONTROL_BPP_24		(0x0000000A)
#define LCD_CONTROL_EN			(0x00000001)

#define LCD_STATUS_MBERROR		(0x00000010)
#define LCD_STATUS_VCOMP		(0x00000008)
#define LCD_STATUS_LNBU			(0x00000004)

#define LCD_INTR_MBERROR		(0x00000010)
#define LCD_INTR_VCOMP			(0x00000008)
#define LCD_INTR_LNBUINTR		(0x00000004)

#define LCD_UPCURR_V(x)			(((x)&0xFFFFFFFF))

#define LCD_LPCURR_V(x)			(((x)&0xFFFFFFFF))

#endif /* ifndef __ASM_ARCH_REGSLCD_H */
