/*
 * include/asm-arm/arch-ns9xxx/regs-rtc.h
 *
 * Copyright (C) 2006 by Digi International Inc.
 * All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License version 2 as published by
 * the Free Software Foundation.
 */
#ifndef __ASM_ARCH_REGSRTC_H
#define __ASM_ARCH_REGSRTC_H

#include <asm/arch/hardware.h>

/* RTC Module */

/* RTC Module base addresses and offsets */
#define RTC_MODULE_BASE_PA	(0x90700000)
#define RTC_MODULE_BASE_VA	io_p2v(RTC_MODULE_BASE_PA)


/* RTC General Control Register */
#define RTC_CGR			__REG(0x90700000)

/* RTC 12/24 Hour Register */
#define RTC_12_24		__REG(0x90700004)

/* RTC Time Register */
#define RTC_TIME		__REG(0x90700008)

/* RTC Calendar Register */
#define RTC_CAL			__REG(0x9070000c)

/* RTC Time Alarm Register */
#define RTC_TAL			__REG(0x90700010)

/* RTC Calendar Alarm Register */
#define RTC_CALAL		__REG(0x90700014)

/* RTC Alarm Enable Register */
#define RTC_ALEN		__REG(0x90700018)

/* RTC Event Flags Register */
#define RTC_EVN			__REG(0x9070001c)

/* RTC Interrupt Enable Register */
#define RTC_IEN			__REG(0x90700020)

/* RTC Interrupt Disable Register */
#define RTC_IDIS		__REG(0x90700024)

/* RTC Interrupt Status Register */
#define RTC_ISTAT		__REG(0x90700028)

/* RTC General Status Register */
#define RTC_GSTAT		__REG(0x9070002c)






/* XXX Adapt to use hardware.h macros */
/* Register bit fields */
#define RTC_GCR_TIME_MASK		(0x00000001)
#define RTC_GCR_TIME_DIS		(0x00000001)
#define RTC_GCR_TIME_EN			(0x00000000)
#define RTC_GCR_CAL_MASK		(0x00000002)
#define RTC_GCR_CAL_DIS			(0x00000002)
#define RTC_GCR_CAL_EN			(0x00000000)

#define RTC_12_24_MODE_MASK		(0x00000001)
#define RTC_12_HOUR_EN			(0x00000001)
#define RTC_24_HOUR_EN			(0x00000000)

#define	RTC_TIME_HOUR_MASK		(0x3f000000)
#define	RTC_TIME_SEC_MASK		(0x00007f00)
#define	RTC_TIME_MIN_MASK		(0x007f0000)

#define	RTC_CAL_DAY_MASK		(0x00000007)
#define	RTC_CAL_MONTH_MASK		(0x000000f8)
#define	RTC_CAL_MDAY_MASK		(0x00003f00)
#define	RTC_CAL_YEAR_MASK		(0x00ff0000)
#define	RTC_CAL_CENTURY_MASK		(0x3f000000)

#define	RTC_AEN_MASK			(0x0000003f)
#define	RTC_AEN_HSEC_EN			(0x00000001)
#define	RTC_AEN_SEC_EN			(0x00000002)
#define	RTC_AEN_MIN_EN			(0x00000004)
#define	RTC_AEN_HOUR_EN			(0x00000008)
#define	RTC_AEN_DATE_EN			(0x00000010)
#define	RTC_AEN_MONTH_EN		(0x00000020)

#define	RTC_EVN_MASK			(0x0000007f)
#define	RTC_EVN_HSEC			(0x00000001)
#define	RTC_EVN_SEC			(0x00000002)
#define	RTC_EVN_MIN			(0x00000004)
#define	RTC_EVN_HOUR			(0x00000008)
#define	RTC_EVN_DATE			(0x00000010)
#define	RTC_EVN_MONTH			(0x00000020)
#define	RTC_EVN_ALARM			(0x00000040)

#define	RTC_IEN_MASK			(0x0000007f)
#define	RTC_IEN_HSEC_EN			(0x00000001)
#define	RTC_IEN_SEC_EN			(0x00000002)
#define	RTC_IEN_MIN_EN			(0x00000004)
#define	RTC_IEN_HOUR_EN			(0x00000008)
#define	RTC_IEN_DATE_EN			(0x00000010)
#define	RTC_IEN_MONTH_EN		(0x00000020)
#define	RTC_IEN_ALARM_EN		(0x00000040)

#define	RTC_IDIS_MASK			(0x0000007f)
#define	RTC_IDIS_HSEC_DIS		(0x00000001)
#define	RTC_IDIS_SEC_DIS		(0x00000002)
#define	RTC_IDIS_MIN_DIS		(0x00000004)
#define	RTC_IDIS_HOUR_DIS		(0x00000008)
#define	RTC_IDIS_DATE_DIS		(0x00000010)
#define	RTC_IDIS_MONTH_DIS		(0x00000020)
#define	RTC_IDIS_ALARM_DIS		(0x00000040)

#define	RTC_ISTAT_MASK			(0x0000007f)
#define	RTC_ISTAT_HSEC_EN		(0x00000001)
#define	RTC_ISTAT_SEC_EN		(0x00000002)
#define	RTC_ISTAT_MIN_EN		(0x00000004)
#define	RTC_ISTAT_HOUR_EN		(0x00000008)
#define	RTC_ISTAT_DATE_EN		(0x00000010)
#define	RTC_ISTAT_MONTH_EN		(0x00000020)
#define	RTC_ISTAT_ALARM_EN		(0x00000040)

#define	RTC_GSTAT_VALID_TIME_CONF	(0x00000001)
#define	RTC_GSTAT_VALID_CAL_CONF	(0x00000002)
#define	RTC_GSTAT_VALID_TAL_CONF	(0x00000004)
#define	RTC_GSTAT_VALID_CALAL_CONF	(0x00000008)





#endif /* ifndef __ASM_ARCH_REGSRTC_H */
